from apifoncier.config import configure
import apifoncier.cartofriches
import apifoncier.dv3f
import apifoncier.dvf_opendata
import apifoncier.ff
import apifoncier.ind_conso_espace
import apifoncier.ind_prix
